// ANN.CPP
//
// This file has function definitions for the ANN.H file
// Written by Dariusz Piatkowski, January 10 1996.

//define some variables which are global and are to be visible inside these
//class member functions
extern double hidden_learn_rate, steepness;

//calculate the output of the neuron
void neuron::Fire()
{
    input+=bias;
    output=1.0/(1.0+exp(-2*steepness*input));
}

//load up the input value from file to the input layer neuron
void neuron::LoadInput(ifstream &file)
{
    file>>output;
}

//load up the output value from the file to the output layer neuron
void neuron::LoadOutput(ifstream &file)
{
    file>>desired_output;
}

//adjust the neuron bias
void neuron::AdjustBias(double learn_rate)
{
    bias+=learn_rate*neuron_error;
}

//randomize the bias value for the neuron
void neuron::Randomize()
{
    int num;

    num=rand()%100;
    bias=2*((double) num/100.00)-1;
}

//calculate the neuron error, overloaded function for the calculation of the
//output layer errors
double neuron::CalcError(int output_layer)
{
    real_error=desired_output-output;
    neuron_error=2*steepness*output*(1-output)*real_error;
    return (real_error*real_error);
}

//calculate the neuron error, overloaded function for the calculation of the
//error for a hidden layer neuron
void neuron::CalcError()
{
    neuron_error=2*steepness*output*(1-output)*real_error;
}

//initalize the neuron variables
void neuron::Initialize(double moment)
{
    good=1;
    input=0.0;
    output=0.0;
    desired_output=0.0;
    last_output=0.0;
    real_error=0.0;
    neuron_error=0.0;
    momentum=moment;
    last_momentum=0.0;
    bias=0.0;
}

//reset some neuron values for another run
void neuron::Reset()
{
    good=1;
    input=0.0;
    real_error=0.0;
    neuron_error=0.0;
}

//initialize the synapse variables
void synapse::Initialize(double moment)
{
    weight=0.0;
    delta_weight=0.0;
    momentum=moment;
}

//generate random weight values
void synapse::Randomize()
{
    int num;

    num=rand()%100;
    weight=2*((double)num/100.00)-1;
}

//pass the value from the "from" neuron to the "to" neuron through the weight
//coefficient of the synapse
void synapse::PassValue()
{
    to->input+=from->output*weight;
}

//calculate the error comitted on a given synapse connection
void synapse::CalcError()
{
    from->real_error+=to->neuron_error*weight;
}

//adjust the synapse weight
void synapse::AdjustWeight(double learn_rate)
{
    double result=0.0;

    result=learn_rate*from->output*to->neuron_error;
    weight+=result+delta_weight;
    delta_weight=momentum*result;
}

