// ANN.H
//
// This is a header file for a neural network.
// It defines classes which are to be used to create: neurons, synapses,
// and networks
// Written by Dariusz Piatkowski, January 8, 1996.

#include <math.h>

class neuron
{
    public:
        void Fire();
        void Reset();
        void Randomize();
        void CalcError();
        void Initialize(double moment);
        double CalcError(int output_layer);
        void AdjustBias(double learn_rate);
        void LoadInput(ifstream &file);
        void LoadOutput(ifstream &file);
        friend class synapse;
        friend void test_net(void);
        friend void clean_up(int argc);
        friend void initialize(int argc, char *argv[]);
        friend int main(int argc, char *argv[]);
    private:
        int good;
        double input;
        double output;
        double last_output;
        double desired_output;
        double real_error;
        double neuron_error;
        double momentum;
        double last_momentum;
        double bias;
};

class synapse
{
    public:
        void Randomize();
        void PassValue();
        void CalcError();
        void Initialize(double moment);
        void AdjustWeight(double learn_rate);
        friend void clean_up(int argc);
        friend void initialize(int argc, char *argv[]);
        friend int main(int argc, char *argv[]);
    private:
        double weight;
        double delta_weight;
        double momentum;
        neuron *from;
        neuron *to;
};



